#!/usr/bin/perl

use strict;
use DBI;
use Data::Dumper;
use Time::ParseDate;

my $database = shift @ARGV or die "Usage: $0 database\n";

my $dbh=DBI->connect("dbi:Pg:dbname=$database","","");
$dbh->do("set client_encoding=UTF8") or die;
$dbh->{AutoCommit}=0;

my $comments = $dbh->selectall_hashref("select * from lj_comments where cid is null and nid is not null",'lj_commentid');

for my $ljc (sort {$a<=>$b} keys %$comments)
  {
    my $cid = insert_comment($comments->{$ljc});
    if ($cid)
      {
        $dbh->do("update lj_comments set cid=? where lj_commentid=?",undef,$cid,$ljc);
      }
  }
$dbh->commit;
$dbh->disconnect;


sub insert_comment
  {
    my $comment = shift or return undef;

    #  parent 
    $comment->{pid} = undef;
    if ($comment->{lj_parentid})
      {
        my $pid = $comment->{lj_parentid};
        while (1)
          {
            my $up = $dbh->selectrow_hashref("select cid,lj_parentid,lj_state from lj_comments where lj_commentid=?",
                                             undef,$pid);
            unless ($up->{lj_state} eq 'D')
              {
                $comment->{pid}=$up->{cid};
                last;
              }
            $pid = $up->{lj_parentid};
            last unless $pid;
          }
      }
    if ($comment->{pid})
      {
        my $pthread = $dbh->selectrow_array("select thread from comments where cid=?",undef,$comment->{pid});
        $pthread=~s|/$||;
        my $max = $dbh->selectrow_array("select max(thread) from comments where nid=? and thread LIKE \'$pthread.\%\'",
                                        undef,$comment->{nid});
        if ($max)
          {
            $max=~s|/$||;
            my @parts = split(/\./,$max);
            my @parent_parts = split(/\./,$pthread);
            my $last = $parts[$#parent_parts];
            $comment->{thread} = $pthread . "." . int2vancode(vancode2int($last)+1)."/";
          }
        else
          {
            $comment->{thread}=$pthread . '.' . int2vancode(0)."/";
          }
        # calculate thread id
      }
    else
      {
          my $max = $dbh->selectrow_array("SELECT MAX(thread) FROM comments WHERE nid = ?", undef, $comment->{nid});
          $max=~s|/$||;
          $comment->{thread} = int2vancode(vancode2int($max) + 1) .'/';
      }

    if ($comment->{uid}==0)
      {
        $comment->{homepage} = $comment->{lj_poster}.".livejournal.com";
      }
    elsif ($comment->{uid}<0)
      {
        $comment->{uid}=0;
        $comment->{lj_poster}=undef;
      }

    $comment->{time} = parsedate($comment->{lj_date});
    $comment->{pid} ||=0;

    $dbh->do("INSERT INTO comments (nid, pid, uid, subject, comment, format, hostname, timestamp, status, thread, name,  homepage) VALUES (?,?,?,?,?,?,?,?,?,?,?,?)", undef,
             $comment->{nid},$comment->{pid},$comment->{uid},$comment->{lj_subject},$comment->{lj_comment},
             0,'127.0.0.1',$comment->{time},0,$comment->{thread},$comment->{lj_poster},$comment->{homepage})
      or die;
    my $cid = $dbh->selectrow_array("SELECT CURRVAL('comments_cid_seq')");
    $comment->{cid}=$cid;

    $dbh->do("update node_comment_statistics set last_comment_timestamp=?,last_comment_name=?,last_comment_uid=?,comment_count = comment_count+1 where nid=?",undef,$comment->{time},$comment->{lj_poster},$comment->{uid},$comment->{nid});
    return $cid;
  }

sub base36
  {
    my $i=shift || 0;
    my @nums=(0..9,'a'..'z')[0..35];
    my $base = 36;
    return $nums[0] if $i == 0;
    my $rep="";
    while ($i>0)
      {
        $rep = $nums[$i%$base].$rep;
        $i = int($i/$base);
      }
    return $rep;
  }

sub frombase36
  {
    my $rep=shift || "";
    my @nums=(0..9,'a'..'z')[0..35];
    my $index = 0;
    my %nums = map {$_,$index++} @nums;
    my $base = 36;
    my $number=0;
    for (split(//,$rep))
      {
        $number *= $base;
        $number += $nums{$_};
      }
    return $number;
  }

sub vancode2int
  {
    my $code = shift || "0";
    return frombase36(substr($code,1));
  }

sub int2vancode
  {
    my $i = shift || 0;
    my $num = base36(int($i));
    my $length = length($num);
    return chr($length + ord('0') - 1) . $num;
}

