#!/usr/bin/perl

use strict;
use LWP::UserAgent;
#use LWP::Debug qw(+);
use DBI;
use Data::Dumper;
use Digest::MD5 qw(md5_hex);
use XML::Simple;
use Encode;
#==
use constant LOGIN_URL => 'http://www.livejournal.com/login.bml';
use  constant COMMENTS_URL => 'http://www.livejournal.com/export_comments.bml';
# == const-end ==

my ($database,$lj_login,$lj_password) = @ARGV;

die "Usage $0 database login password" unless  $lj_login && $lj_password;

my $chal;
my $ua = new LWP::UserAgent;
$ua->requests_redirectable(['GET','HEAD','POST']);
$ua->agent("Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.19) Gecko/7 Firefox/3.0.0.11");
$ua->cookie_jar ({});
my $resp = $ua->get(LOGIN_URL);
die "cannot get challenge" unless $resp->is_success;
my $resp_str = $resp->content;

if ($resp_str=~m|<input[^>]+name=\'chal\'[^>]+value=\'([^']+)\'|)
  {
    $chal = $1;
  } 
my $fauf;
if ($resp_str=~m|<input[^>]+name=\"lj_form_auth\"[^>]+value=\"([^']+)\"|)
  {
    $fauf = $1;
  } 
die "No challenge received" unless $chal;

my %form;

$form{chal}=$chal;
$form{lj_form_auth}=$fauf;
$form{user}=$lj_login;
$form{password}="";
$form{response}=md5_hex($chal.md5_hex($lj_password));
$form{'action:login'}='Log in...';

$ua->default_header(Referer=>LOGIN_URL);
#$ua->add_handler("request_send",  sub { shift->dump; return });
#$ua->add_handler("response_done", sub { shift->dump; return });
$resp = $ua->post(LOGIN_URL,\%form);
die "cannot get login cookies" unless $resp->is_success;

my $dbh=DBI->connect("dbi:Pg:dbname=$database","","");
$dbh->do("set client_encoding=UTF8") or die;
$dbh->{AutoCommit}=0;
my @data = $dbh->selectrow_array("select max(lj_commentid) from lj_comments",undef);

my $maxid = $data[0]+1 || 0;

$resp = $ua->post(COMMENTS_URL."?get=comment_meta&startid=$maxid");
die "cannot get comment_meta" unless $resp->is_success;

my $meta_data = XMLin($resp->content);

my $uidmap;
my $uid2name;

my $umap = $meta_data->{usermaps}->{usermap};
if (exists $umap->{id})
  {
    $umap = {$umap->{id}=>$umap};
  }

for my $uid(keys %{$umap})
  {
    $uidmap->{$uid}=0;
    $uidmap->{$uid} = 1 if $meta_data->{usermaps}->{usermap}->{$uid}->{user} eq $lj_login;
    $uid2name->{$uid} = $meta_data->{usermaps}->{usermap}->{$uid}->{user};
  }

$uid2name->{-1}='Anonymous';
$uidmap->{-1}=-1;

$resp = $ua->post(COMMENTS_URL."?get=comment_body&startid=$maxid");
die "get comment_body" unless $resp->is_success;


my $comments=XMLin($resp->content);
my $comment_data=parse_comments($comments);

my $ljmapping = $dbh->selectall_hashref("select ljid,nid from ljxp_node",'ljid',undef);

my $updcomment = $dbh->prepare(
"update lj_comments set lj_date=?,lj_jitemid=?,lj_parentid=?,lj_state=?,lj_poster=?,".
                               "lj_subject=?,lj_comment=?,nid=?,uid=? where lj_commentid=?"
                              );
my $inscomment = $dbh->prepare(
                               "insert into lj_comments(lj_date,lj_jitemid,lj_parentid,lj_state,lj_poster,".
                               "lj_subject,lj_comment,uid,nid,lj_commentid) values(?,?,?,?,?,?,?,?,?,?)"
                              );
my $updsubj = $dbh->prepare("update lj_comments set lj_subject = substring(lj_comment from 1 for 60) where (length(lj_subject)<1 or  lj_subject is null) and lj_commentid=?");

my ($new,$up)=(0,0);
foreach my $cmn(@{$comment_data})
  {
    next unless $cmn->{id}; # unknown id ??
    next unless exists $ljmapping->{$cmn->{jitemid}};
    my $nid = $ljmapping->{$cmn->{jitemid}};

    $cmn->{posterid}||=-1;
    $cmn->{poster} = $uid2name->{$cmn->{posterid}};
    $cmn->{state} ||= ' '; #
    my $uid = $uidmap->{$cmn->{posterid}};
    $uid = -1 unless exists $uidmap->{$cmn->{posterid}};

    my $rows;
      if (($rows = $updcomment->execute($cmn->{date},$cmn->{jitemid},$cmn->{parentid},$cmn->{state},$cmn->{poster},
                                        $cmn->{subject}, $cmn->{body},$uid, $nid->{nid},$cmn->{id}))=!1)
        {
          $new++;
          $rows = $inscomment->execute($cmn->{date},$cmn->{jitemid},$cmn->{parentid},$cmn->{state},$cmn->{poster},
                                       $cmn->{subject},$cmn->{body}, $uid,$nid->{nid},$cmn->{id});
          $updsubj->execute($cmn->{id});

        }
    else
      {
        $up++;
      }

    die "Cannot insert/update comments" unless $rows == 1;
  }
#print "$new new, $up updated\n";
$inscomment->finish;
$updcomment->finish;

$dbh->commit;
$dbh->disconnect;
exit 0;

sub parse_comments
{
  my $ret = [];
  my $data = shift or return [];
  my $enc=shift || 'koi8-r';
  my $cdata = $data->{comments}->{comment};
  return [] unless $cdata;

  if (exists $cdata->{id})
    {
      $cdata = {$cdata->{id}=>$cdata};
    }

  foreach my $id (keys %$cdata)
  {
     Encode::_utf8_off($id);
     my $item={};
     $item->{id}=$id;
     foreach my $key(keys %{$cdata->{$id}})
     {
	Encode::_utf8_off($key);
	$item->{$key} = encode_utf8(decode_utf8($cdata->{$id}->{$key}));
	Encode::_utf8_off($item->{$key});
     }
     push @$ret,$item;
  }
  return $ret;
}

